@@ -27,8 +27,9 @@ function (object, process = c("Longitudinal", "longitudinal", "Event", "event"),
         formYx <- reformulate(attr(delete.response(TermsX), "term.labels"))
         formYz <- Forms$formYz
         times <- Data$data[[timeVar]]
-        wk <- gaussKronrod()$wk
-        sk <- gaussKronrod()$sk
+        GQsurv <- if (object$control$GQsurv == "GaussKronrod") gaussKronrod() else gaussLegendre(object$control$GQsurv.k)
+        wk <- GQsurv$wk
+        sk <- GQsurv$sk
         K <- length(sk)
         P <- times/2
         st <- outer(P, sk + 1)
